//Krb4SecureTCPClient.java

import java.io.*;
import java.net.*;
import java.util.*;
import krb4.lib.*;
import krb4.lib.crypto.*;

class Krb4SecureTCPClient {

	public static void main(String args[]) throws UnknownHostException {
		String cname = "ns10-demo";
		String cinst = "";
		String crealm = "CIT.CORNELL.EDU";
		String sname = "tt10-demo";
		String sinst = "";
		String srealm = "CIT.CORNELL.EDU";
		byte[] key = des.string_to_key_bytes("$tester");
		String kdc = "kerberos.cit.cornell.edu";
		InetAddress address = InetAddress.getLocalHost();
		int port = 8895;
		TCPClient client = null;

		try {

			//as_req

			Krb4ASReq as_req = new Krb4ASReq(
				cname,
				cinst,
				crealm,
				srealm
			);

			System.out.println("send as_req");

			as_req.send(kdc);
			Krb4ASRep as_rep = as_req.getKrb4ASRep();
			Krb4Creds initial_creds = as_rep.getCreds(key);

			System.out.println("received as_rep");

			//tgs_req

			Krb4TGSReq tgs_req = new Krb4TGSReq(
				initial_creds,
				cname,
				cinst,
				crealm,
				sname,
				sinst,
				srealm
			);
			System.out.println("send tgs_req");

			tgs_req.send(kdc);
			Krb4TGSRep tgs_rep = tgs_req.getKrb4TGSRep();
			Krb4Creds creds = tgs_rep.getCreds(initial_creds);

			System.out.println("received tgs_rep");
			System.out.println();
			client = new TCPClient(address, port);

			String str = " 01234567ABCDEFGHello; this is a secure message!";
			byte[] userData = new byte[str.length()];
			str.getBytes(0, str.length(), userData, 0);

			Krb4APReq ap_req = new Krb4APReq(
				creds,
				cname,
				cinst,
				crealm,
				false
			);

			System.out.println("send ap_req");

			client.send(ap_req.encode());
			byte[] ibuf = client.receive();

			System.out.println("received ap_rep");
			
			Krb4APRep ap_rep = new Krb4APRep(ibuf);
			ap_rep.authenticate(creds, ap_req);

			System.out.println("authenticated");
			System.out.println();

			do {

				System.out.println("userData=\"" + new String(userData, 0) + "\"");

				Krb4KRBPriv krb_priv = new Krb4KRBPriv(
					creds,
					userData,
					InetAddress.getLocalHost().getAddress(),
					client.getLocalPort(),
					client.getInetAddress().getAddress(),
					client.getPort()
				);

				System.out.println("send");

				client.send(krb_priv.encode());
				ibuf = client.receive();

				System.out.println("received");
				
				Krb4KRBPriv krb_priv_reply = new Krb4KRBPriv(ibuf);

				byte[] replyData = krb_priv_reply.decrypt(creds);

				System.out.println("decrypted");

				System.out.println("replyData=\"" + new String(replyData, 0) + "\"");
				System.out.println();

				//why wait..
				try {
					Thread.currentThread().sleep(1000);
				} catch (InterruptedException e) { }

			}
			while (true);
		}
		catch (Krb4Exception e) {
			e.printStackTrace();
			System.out.println(e.krbErrorMessage());
		}
		catch (Exception e) {
			e.printStackTrace();
		}
		finally {
			try {
				if (client != null)
					client.close();
			}
			catch (IOException e) {
			}
		}		

		System.out.println("ok");
	}
}