//Krb4SecureTCPServer.java

import java.io.*;
import java.net.*;
import java.util.*;
import krb4.lib.*;
import krb4.lib.crypto.*;

class Krb4SecureTCPServer extends Thread {
	TCPClient client;
	static byte[] key = des.string_to_key_bytes("$tester");
	static String sname = "tt10-demo";
	static String sinst = "";
	static String srealm = "CIT.CORNELL.EDU";
	static int port = 8895;

	public Krb4SecureTCPServer(Socket newSocket) throws IOException {
		client = new TCPClient(newSocket);
	}

	public static void main(String[] args) throws IOException {
		ServerSocket server = new ServerSocket(port);
		try {
			System.out.println("listening...");
			do {
				Krb4SecureTCPServer client =
					new Krb4SecureTCPServer(server.accept());
				client.start();
			}
			while(true);
		}
		catch (IOException e) {
			e.printStackTrace();
		}
		finally {
			try {
				server.close();
			}
			catch (IOException e) {
			}
		}
	}

	public void run() {
		try {
			byte[] ibuf = client.receive();
			if (ibuf == null || ibuf.length == 0)
				//client disconnected
				return;

			System.out.println("received ap_req");

			Krb4APReq ap_req = new Krb4APReq(ibuf);
			ap_req.authenticate(
				key,
				sname,
				sinst,
				srealm,
				client.getInetAddress().getAddress()
			);
			System.out.println("authenticated");
                        
            String cname = ap_req.getClientName();
            String cinst = ap_req.getClientInstance();
            String crealm = ap_req.getClientRealm();
			System.out.print("client = " + cname);
			if (cinst != null && cinst.length() > 0)
				System.out.print("/" + cinst);
			System.out.println("@" + crealm);
			System.out.println();

			Krb4APRep ap_rep = new Krb4APRep(
				ap_req,
				InetAddress.getLocalHost().getAddress(),
				client.getLocalPort(),
				client.getInetAddress().getAddress(),
				client.getPort()
			);

			System.out.println("sending ap_rep");
			client.send(ap_rep.encode());

			do {
				ibuf = client.receive();
				if (ibuf == null || ibuf.length == 0)
					//client disconnected
					return;

				System.out.println("received");

				Krb4KRBPriv krb_priv = new Krb4KRBPriv(ibuf);

				byte[] userData = krb_priv.decrypt(ap_req);

				System.out.println("decrypted");

				System.out.println("userData=\"" +
					new String(userData, 0) + "\"");

				String str = "<" + new String(userData, 0)
					+ ">";
				byte[] userData2 = new byte[str.length()];
				str.getBytes(0, str.length(), userData2, 0);
				
				Krb4KRBPriv krb_priv_reply = new Krb4KRBPriv(
					ap_req,
					userData2,
					InetAddress.getLocalHost().getAddress(),
					client.getLocalPort(),
					client.getInetAddress().getAddress(),
					client.getPort()
				);

				System.out.println("sending");
				client.send(krb_priv_reply.encode());
				System.out.println();
			}
			while(true);
		}
		catch (IOException e) {
		}
		catch (Krb4Exception e) {
			e.printStackTrace();
			System.out.println(e.krbErrorMessage());
		}
		catch (Exception e) {
			e.printStackTrace();
		}
		finally {
			try {
				client.close();
			}
			catch (IOException e) {
			}
		}
	}

}
