import java.io.*;
import java.util.*;
import krb5.lib.*;
import krb5.lib.util.*;
import krb5.lib.ccache.*;
import krb5.lib.crypto.*;

class jklist {
    
    static void printUsage() {
        System.out.println("Usage: java jklist -h | -? | [-f] [-e] [-k] [-s] [cache_name]");
        System.out.println("\t-h and -? display help");
        System.out.println("\t-f shows credentials flags");
        System.out.println("\t-e shows the encryption type");
        System.out.println("\t-k shows the encryption key");
        System.out.println("\t-s sets exit status based on valid tgt existence");
    }

	public static void main(String args[]) {
	    boolean showCredentialsFlags = false;
	    boolean showEncryptionType = false;
	    boolean showEncryptionKey = false;
	    boolean setExitStatus = false;
	    String name = null;
        boolean validTGT = true;

	    Getopt getopt = new Getopt("efhks?", args);
	    if (getopt.invalidOptions()) {
	            System.out.println("jklist: invalid option");
	            printUsage();
	            return;
	    }
	    
	    Dictionary opts = getopt.getOptTable();
	    Enumeration optsEnum = opts.keys();
	    while (optsEnum.hasMoreElements()) {
	        String opt = (String)optsEnum.nextElement();
	        if (opt.equals("h") || opt.equals("?")) {
	            printUsage();
	            return;
	        }
	        if (opt.equals("f"))
	            showCredentialsFlags = true;
	        if (opt.equals("e"))
	            showEncryptionType = true;
	        if (opt.equals("k"))
	            showEncryptionKey = true;
	        if (opt.equals("s"))
	            setExitStatus = true;
	    }

	    String[] trailingArgs = getopt.getTrailingArgs();
	    if (trailingArgs.length > 1) {
	        System.out.println("jklist: too many args");
	        printUsage();
	        return;
	    }
	    if (trailingArgs.length > 0)
	        name = trailingArgs[0];

	    try {
	        if (name == null)
	            name = Config.getDefaultCredentialsCacheName();
	        System.out.println("Ticket cache: " + name);
	        String defaultPrincipal = "";
	        CredentialsCache ccache = Config.getCredentialsCache(name);
	        PrincipalName principalName = ccache.getPrimaryPrincipal();
	        if (principalName != null)
	            defaultPrincipal = principalName.toString();
	        
	        System.out.println("Default principal: " + defaultPrincipal);
	        System.out.println();
	        System.out.println("Valid starting\tExpires\t\tService principal");
	        
            if (principalName == null) {
                if (setExitStatus)
                    System.exit(1);
                return;
            }
            
            if (setExitStatus) {
                //XXX eventually change to ensure user is not prompted here
                Credentials initialCreds = ccache.lookupSingleValidCredentials(
                    ccache.getTGTServiceName());
                if (initialCreds != null)
                    validTGT = true;
            }
            
	        CCacheEnumerator credsEnum = ccache.lookupCredentials(
	            new MatchPrincipalCredsFilter(principalName));
	        while (credsEnum.hasMoreCredentials()) {
	            Credentials creds = credsEnum.nextCredential();
                
                String starting;
                if (creds.starttime != null)
                    starting = creds.starttime.toString();
                else
                    starting = creds.authtime.toString();
                
                String expires = creds.endtime.toString();
                
                String servicePrincipal = creds.server().toString();
                
	            System.out.println(starting + "\t" + expires + "\t" + servicePrincipal);
	            if (showCredentialsFlags || showEncryptionType)
	                System.out.print("\t");
	            if (showCredentialsFlags)
	                System.out.print("Flags: " + creds.flags.toString());
	            if (showEncryptionType) {
	                if (showCredentialsFlags)
	                    System.out.print(", ");
	                System.out.print("Etype: " + EType.toString(creds.key.keyType));
	            }
	            if (showCredentialsFlags || showEncryptionType)
	                System.out.println();
	            if (showEncryptionKey)
	                System.out.println("\tKey: " + SysUtil.byte2hexString(creds.key.keyValue));
	        }
	        
	    } catch (KrbException e) {
	        System.out.println("jklist: " + e.krbErrorMessage());
	    }
        if (setExitStatus) {
            if (validTGT)
                System.exit(0);
            else
                System.exit(1);
        }
	}
}
