// Copyright 1997 The Open Group Research Institute.  All rights reserved.

package krb5.lib;

import krb5.lib.asn1.*;
import java.util.Vector;

/**
 * Encryption type information
 */
public class ETypeInfo {

	/**
	 * Encryption info entry
	 */
	public ETypeInfoEntry[] entry;

	/**
	 * Class constructor
	 *
	 * @param ref is of type EncodeRef
	 * @exception Asn1Exception an exception
	 * @see krb5.lib.asn1.EncodeRef
	 * @see krb5.lib.Asn1Exception
	 */
	public ETypeInfo(EncodeRef ref) throws Asn1Exception {
		Vector tempEntry = new Vector();
		EncodeRef subRef = decode.Sequence(ref);
		while (subRef.isMoreInSequence()) {
			tempEntry.addElement(new ETypeInfoEntry(subRef));
			subRef.next();
		}
		if (tempEntry.size() > 0) {
			entry = new ETypeInfoEntry[tempEntry.size()];
			tempEntry.copyInto(entry);
		}
	}

	/**
	 * Encodes the object in asn1
	 *
	 * @return byte[] is a result
	 * @exception Asn1Exception an exception
	 * @see krb5.lib.Asn1Exception
	 */
	public byte[] asn1Encode() throws Asn1Exception {
		byte[][] etype_info = null;
		if (entry != null && entry.length > 0) {
			etype_info = new byte[entry.length][];
			for (int i = 0; i < entry.length; i++)
				etype_info[i] = entry[i].asn1Encode();
		}
		return encode.Sequence(etype_info);
	}

}