// Copyright 1997 The Open Group Research Institute.  All rights reserved.
// Copyright 1999 Memesoft.  All rights reserved.

package krb5.lib.ccache;

import krb5.lib.*;

/**
 * Unexpired credentials filter
 */
public class UnexpiredCredsFilter implements CredentialsFilter {

	/**
	 * Consider Skew ?
	 */
    boolean considerSkew;

	/**
	 * Time
	 */
    KerberosTime time;

	/**
	 * Class constructor
	 */
    public UnexpiredCredsFilter() {
        considerSkew = true;
    }

	/**
	 * Class constructor
	 *
	 * @param _considerSkew is of type boolean
	 */
    public UnexpiredCredsFilter(boolean _considerSkew) {
        considerSkew = _considerSkew;
    }

	/**
	 * Class constructor
	 *
	 * @param _considerSkew is of type boolean
	 * @param _time is of type KerberosTime
	 * @see krb5.lib.KerberosTime
	 */
    public UnexpiredCredsFilter(boolean _considerSkew, KerberosTime _time) {
        considerSkew = _considerSkew;
        time = (KerberosTime)_time.clone();
    }

	/**
	 * Filters
	 *
	 * @return boolean is a result
	 * @param cred is of type Credentials
	 * @see krb5.lib.ccache.Credentials
	 */
    public boolean filter(Credentials cred) {
        KerberosTime now = time;
        boolean result;

        if (now == null)
            now = new KerberosTime(true);

        if (considerSkew)
            result = cred.endtime.greaterThanWRTClockSkew(now);
        else
            result = cred.endtime.greaterThan(now);

        return result;
    }
}