// Copyright 1997 The Open Group Research Institute.  All rights reserved.

package krb5.lib.crypto;

import krb5.lib.*;

public abstract class DesCbcEType extends EType {

	public int blockSize() {
		return 8;
	}

	public int keyType() {
		return Krb5.KEYTYPE_DES;
	}

	public int keySize() {
		return 8;
	}

	//XXX for testing...
	//public static final byte[] Confounder2 = {
	//	(byte)0x70, (byte)0x22, (byte)0xD0, (byte)0x28, (byte)0xA6, (byte)0x6E, (byte)0xB3, (byte)0x52
	//};

	public byte[] encrypt(byte[] data, byte[] key) {
		long[] key_sched = new long[16];
		des.des_set_key(key, key_sched);
		byte[] ivec = new byte[keySize()];
		int new_size = data.length + confounderSize() +	checksumSize();
		byte[] new_data = new byte[new_size +
			(new_size % blockSize() == 0 ? 0 : blockSize() -
			(new_size % blockSize()))];
		byte[] conf = Confounder.bytes(confounderSize());
		System.arraycopy(conf, 0, new_data, 0, confounderSize());
		System.arraycopy(data, 0, new_data, startOfData(), data.length);
		byte[] cksum = calculateChecksum(new_data, new_data.length);
		System.arraycopy(cksum, 0, new_data, startOfChecksum(),
			checksumSize());
		byte[] cipher = new byte[new_data.length];
		//XXX would like it to work in place
		des.cbc_encrypt(new_data, cipher, key_sched, ivec, true);
		return cipher;
	}

	public byte[] encrypt(byte[] data, byte[] key, byte[] ivec) {
		long[] key_sched = new long[16];
		des.des_set_key(key, key_sched);
		int new_size = data.length + confounderSize() +	checksumSize();
		byte[] new_data = new byte[new_size +
			(new_size % blockSize() == 0 ? 0 : blockSize() -
			(new_size % blockSize()))];
		byte[] conf = Confounder.bytes(confounderSize());
		System.arraycopy(conf, 0, new_data, 0, confounderSize());
		System.arraycopy(data, 0, new_data, startOfData(), data.length);
		byte[] cksum = calculateChecksum(new_data, new_data.length);
		System.arraycopy(cksum, 0, new_data, startOfChecksum(),
			checksumSize());
		byte[] cipher = new byte[new_data.length];
		//XXX would like it to work in place
		des.cbc_encrypt(new_data, cipher, key_sched, ivec, true);
		return cipher;
	}

	public void decrypt(byte[] cipher, byte[] key)
		throws KrbApErrException {
		long[] key_sched = new long[16];
		des.des_set_key(key, key_sched);
		byte[] ivec = new byte[keySize()];
		byte[] data = new byte[cipher.length];
		//XXX would like it to work in place
		des.cbc_encrypt(cipher, data, key_sched, ivec, false);
		System.arraycopy(data, 0, cipher, 0, cipher.length);
		if (!isChecksumValid(data))
			throw new KrbApErrException(Krb5.KRB_AP_ERR_BAD_INTEGRITY);
	}

	public void decrypt(byte[] cipher, byte[] key, byte[] ivec)
		throws KrbApErrException {
		long[] key_sched = new long[16];
		des.des_set_key(key, key_sched);
		byte[] data = new byte[cipher.length];
		//XXX would like it to work in place
		des.cbc_encrypt(cipher, data, key_sched, ivec, false);
		System.arraycopy(data, 0, cipher, 0, cipher.length);
		if (!isChecksumValid(data))
			throw new KrbApErrException(Krb5.KRB_AP_ERR_BAD_INTEGRITY);
	}

}
